package com.codigoparallevar.minicards;

import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import com.codigoparallevar.minicards.toolbox.PartsHolder;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.codigoparallevar.minicards.types.functional.Tuple3;
import com.codigoparallevar.minicards.ui_helpers.GetAsync;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.programaker.api.ProgramakerApi;
import com.programaker.api.data.ProgramakerBridgeInfo;
import com.programaker.api.data.api_results.ProgramakerBridgeCustomBlockResult;
import com.programaker.api.data.api_results.ProgramakerGetCustomBlocksResult;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CardActivity extends AppCompatActivity {

    public final static String INTENT = "com.codigoparallevar.minicards.CARD";
    public static final String CARD_PATH_KEY = "CARD_PATH";
    public static final String VISUALIZATION_MODE_KEY = "VISUALIZATION_MODE";
    public static final String DEVELOPER_VISUALIZATION_MODE = "DEVELOPER_VISUALIZATION_MODE";
    public static final String USER_VISUALIZATION_MODE = "USER_VISUALIZATION_MODE";

    private int permissionRequestLatest = 1;
    private Map<Integer, Runnable> permissionRequestCallbacks = new HashMap<>();

    CanvasView canvasView;
    com.getbase.floatingactionbutton.AddFloatingActionButton AddPartButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton SetUserModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton CenterViewButton;

    com.getbase.floatingactionbutton.FloatingActionsMenu devFabMenu;
    com.getbase.floatingactionbutton.FloatingActionsMenu userFabMenu;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromDevModeButton;
    com.getbase.floatingactionbutton.FloatingActionButton ShowDeckFromUserModeButton;

    boolean devMode = false;
    FloatingActionButton removePartFab;
    private PartsHolder partsHolder;
    private ProgramakerApi ProgramakerApi;
    private ConfigManager Config;
    private ProgramakerGetCustomBlocksResult CustomBlocks = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        this.Config = new ConfigManager(this);
        this.ProgramakerApi = new ProgramakerApi();
        String token = this.Config.getToken();
        if (token != null) {
            this.ProgramakerApi.setToken(token);
        }

        partsHolder = new PartsHolder(this);

        new GetAsync<Tuple2<List<ProgramakerBridgeInfo>, List<ProgramakerBridgeCustomBlockResult>>>()
                .executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,
                        new Tuple3<>(() ->
                                new Tuple2<>(
                                        CardActivity.this.ProgramakerApi.fetchConnectedBridges(),
                                        CardActivity.this.ProgramakerApi.fetchCustomBlocks()),
                                result -> {
                                    partsHolder.addCustomBlocks(result.item1, result.item2);
                                    Log.d("CARDActivity", "custom blocks: " + result.toString());
                                },
                                exception -> Log.e("CARDActivity", "error retrieving custom blocks: " + exception.toString())));

        // Hide action bar
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.hide();
        }

        setContentView(R.layout.activity_main);

        // Use manually controlled canvas
        canvasView = (CanvasView) findViewById(R.id.canvasView);
        canvasView.setParentActivity(this);

        // Initialize auxiliary elements

        removePartFab = (FloatingActionButton) findViewById(R.id.remove_part_fab);
        canvasView.setDropZone(
                removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());

        // Set callbacks to fab buttons
        devFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.dev_mode_fab);
        userFabMenu = (com.getbase.floatingactionbutton.FloatingActionsMenu)
                findViewById(R.id.user_mode_fab);

        AddPartButton = (com.getbase.floatingactionbutton.AddFloatingActionButton)
                findViewById(R.id.add_part_button);
        AddPartButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                partsHolder.openAddPartModal(canvasView);
            }
        });

        CenterViewButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.center_view_button);
        CenterViewButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                canvasView.CenterView();
            }
        });

        SetDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_developer_mode_button);
        SetDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(true);
            }
        });

        ShowDeckFromDevModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_dev_mode_button);
        ShowDeckFromDevModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        ShowDeckFromUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.show_deck_from_user_mode_button);
        ShowDeckFromUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        SetUserModeButton = (com.getbase.floatingactionbutton.FloatingActionButton)
                findViewById(R.id.set_user_mode_button);
        SetUserModeButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CardActivity.this.setDevMode(false);
            }
        });

        String visualization_mode = this.getIntent().getStringExtra(VISUALIZATION_MODE_KEY);
        if ((visualization_mode != null) && (visualization_mode.equals(DEVELOPER_VISUALIZATION_MODE))) {
            devMode = true;
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        loadCard();
        canvasView.resume();
    }

    @Override
    protected void onPause() {
        super.onPause();

        canvasView.pause();
    }

    private void loadCard() {
        String cardPath = this.getIntent().getStringExtra(CARD_PATH_KEY);
        if (cardPath == null) {
            finish();
        }

        try {
            canvasView.loadCard(cardPath);
        } catch (IOException e) {
            e.printStackTrace();
            finish();
        } catch (ErrorLoadingCardException e) {
            e.printStackTrace();
            finish();
        }
        setDevMode(devMode);
    }


    private void setDevMode(boolean goToDevMode) {
        boolean willInvalidate = devMode != goToDevMode;

        devMode = goToDevMode;
        if (goToDevMode){
            devFabMenu.setVisibility(View.VISIBLE);
            userFabMenu.setVisibility(View.GONE);
        }
        else {
            devFabMenu.setVisibility(View.GONE);
            userFabMenu.setVisibility(View.VISIBLE);
        }

        this.canvasView.setDevMode(devMode);

        if (willInvalidate){
            invalidate();
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (canvasView != null){
            return canvasView.onTouchEvent(event);
        }

        Log.e("CardActivity", "CanvasView is null");

        return super.onTouchEvent(event);
    }

    public void invalidate() {
        if (canvasView != null){
            if (canvasView.isDragging()){
                devFabMenu.setVisibility(View.GONE);
                userFabMenu.setVisibility(View.GONE);
                ((View)removePartFab).setVisibility(View.VISIBLE);
            }
            else {
                this.setDevMode(devMode);
                ((View)removePartFab).setVisibility(View.GONE);
            }
            canvasView.setDropZone(
                    removePartFab.getX(), removePartFab.getX() + removePartFab.getWidth(),
                    removePartFab.getY(), removePartFab.getY() + removePartFab.getHeight());
        }
    }

    public static void openCard(Context context, CardFile cardfile, String visualizationMode) {
        openCardByPath(context, cardfile.getPath(), visualizationMode);
    }

    public static void openCard(Context context, PreviewCard cardfile, String visualizationMode) {
        openCardByPath(context, cardfile.getPath(), visualizationMode);
    }

    private static void openCardByPath(Context context, String path, String visualizationMode) {
        Intent i = new Intent(INTENT);
        i.putExtra(CARD_PATH_KEY, path);
        if (visualizationMode != null) {
            i.putExtra(VISUALIZATION_MODE_KEY, visualizationMode);
        }
        context.startActivity(i);
    }

    public void requestPermissions(String permission, Runnable ifAccepted) {
        permissionRequestLatest++;
        int request_code = permissionRequestLatest;
        permissionRequestCallbacks.put(request_code, ifAccepted);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            this.requestPermissions(new String[]{permission}, request_code);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        // If request is cancelled, the result arrays are empty.
        if (grantResults.length > 0 &&
                grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            // Permission is granted. Continue the action or workflow
            // in your app.
            if (permissionRequestCallbacks.containsKey(requestCode)) {
                Runnable callback = permissionRequestCallbacks.get(requestCode);
                permissionRequestCallbacks.remove(requestCode);
                callback.run();
            }
        }  else {
            // TODO:
            // Explain to the user that the feature is unavailable because
            // the features requires a permission that the user has denied.
            // At the same time, respect the user's decision. Don't link to
            // system settings in an effort to convince the user to change
            // their decision.
            if (permissionRequestCallbacks.containsKey(requestCode)) {
                permissionRequestCallbacks.remove(requestCode);
            }
        }
    }
}
