package com.codigoparallevar.minicards.bridge;

import android.content.Context;
import android.provider.Settings;

import com.codigoparallevar.minicards.ConfigManager;
import com.codigoparallevar.minicards.bridge.blocks.DefaultAndroidBlocks;
import com.codigoparallevar.minicards.bridge.helpers.StatusStreamer;
import com.programaker.bridge.ProgramakerBridge;
import com.programaker.bridge.ProgramakerBridgeConfiguration;

public class ProgramakerAndroidBridge {
    private static final String LogTag = "PM Android Bridge";
    private ProgramakerBridge bridgeRunner = null;
    private StatusStreamer statusStreamer = null;

    // Static
    public static ProgramakerAndroidBridge configure(Context ctx, String userId, String bridgeId, String bridgeToken) {
        return new ProgramakerAndroidBridge(ctx, userId, bridgeId, bridgeToken);
    }

    public static String GetBridgeName(Context ctx) {
        String deviceName = Settings.Secure.getString(ctx.getContentResolver(), "bluetooth_name");
        String serviceName = "MiniCards on " + deviceName;
        return serviceName;
    }

    // Builder
    private final Context ctx;
    private final String userId;
    private final String bridgeId;
    private final String bridgeToken;

    private ProgramakerAndroidBridge(Context ctx, String userId, String bridgeId, String bridgeToken) {
        this.ctx = ctx;
        this.userId = userId;
        this.bridgeId = bridgeId;
        this.bridgeToken = bridgeToken;
    }

    public void start(Runnable onReady, Runnable onComplete) {
        ProgramakerBridgeConfiguration configuration = new ProgramakerBridgeConfiguration(
                ProgramakerAndroidBridge.GetBridgeName(this.ctx),
                new ConfigManager(this.ctx),
                DefaultAndroidBlocks.GetBuilder(this.ctx).Build()
        );

        this.bridgeRunner = new ProgramakerBridge(this.bridgeId, this.userId, this.bridgeToken, configuration, onReady, onComplete);
        this.bridgeRunner.run();

        this.statusStreamer = new StatusStreamer(this.ctx, this.bridgeRunner);
        this.statusStreamer.run();
    }

    public void stop() {
        bridgeRunner.stop();
        this.statusStreamer.stop();
    }
}
