package com.codigoparallevar.minicards.bridge.blocks;

import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.json.JSONException;
import org.json.JSONObject;

class BlockArgumentDefinition {
    private final static String LogTag = "PM BlockArgument";
    private final Type type;
    private final String defaultValue;

    public enum Type {
        STRING,
        INT,
        FLOAT,
        BOOL,
    }

    BlockArgumentDefinition(BlockArgumentDefinition.Type type, String defaultValue) {
        this.type = type;
        this.defaultValue = defaultValue;
    }

    @NotNull
    public JSONObject serialize() throws JSONException {
        JSONObject obj = new JSONObject();

        obj.put("type", BlockArgumentDefinition.TypeToString(type));
        obj.put("default", this.defaultValue);

        return obj;
    }

    private static String TypeToString(Type type) {
        switch (type) {
            case STRING:
                return "string";
            case INT:
                return "integer";
            case FLOAT:
                return "float";
            case BOOL:
                return "bool";
            default:
                Log.e(LogTag, "Unknown type: " + type);
                return "string";
        }
    }
}
