package com.codigoparallevar.minicards.bridge.blocks;

import com.codigoparallevar.minicards.types.functional.Function;
import com.programaker.bridge.ProgramakerBridgeConfigurationBlock;

import java.util.LinkedList;
import java.util.List;

public class BridgeBlockListBuilder {
    private final LinkedList<ProgramakerBridgeConfigurationBlock> blockList;

    public BridgeBlockListBuilder() {
        this.blockList = new LinkedList<>();
    }


    public List<ProgramakerBridgeConfigurationBlock> Build() {
        return blockList;
    }

    public BridgeBlockListBuilder addOperation(String id, String message,
                                               List<BlockArgumentDefinition> arguments,
                                               Function<List<?>, Void> operation) {
        return this.addOperation(new OperationBlockDefinition(id, message, arguments, operation));
    }

    private BridgeBlockListBuilder addOperation(ProgramakerBridgeConfigurationBlock block) {
        this.blockList.add(block);
        return this;
    }

    public BridgeBlockListBuilder addGetter(String id, String message,
                                            List<BlockArgumentDefinition> arguments,
                                            Function<List<?>, ?> operation) {
        return this.addGetter(new GetterBlockDefinition(id, message, arguments, operation));
    }

    private BridgeBlockListBuilder addGetter(ProgramakerBridgeConfigurationBlock block) {
        this.blockList.add(block);
        return this;
    }

    public BridgeBlockListBuilder addTrigger(String id, String message,
                                             List<BlockArgumentDefinition> arguments,
                                             String key) {
        return this.addTrigger(new TriggerBlockDefinition(id, message, arguments, key));
    }

    private BridgeBlockListBuilder addTrigger(TriggerBlockDefinition block) {
        this.blockList.add(block);

        return this;
    }
}
