package com.codigoparallevar.minicards.parts.connectors;

import android.util.Log;

import com.codigoparallevar.minicards.CanvasView;
import com.codigoparallevar.minicards.parts.values.StaticValuePart;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.connectors.RoundConnector;
import com.codigoparallevar.minicards.types.connectors.Wiring.AnyWire;
import com.codigoparallevar.minicards.types.connectors.Wiring.Wire;
import com.codigoparallevar.minicards.types.connectors.input.AnyInputConnector;
import com.codigoparallevar.minicards.types.wireData.AnySignal;

import java.util.LinkedList;
import java.util.List;

public class ProgramakerCustomBlockInputConnector extends AnyInputConnector implements RoundConnector {
    private static final int STATIC_VALUE_SEPARATION = 200;
    private final Part _part;
    private final ConnectorTypeInfo _typeInfo;
    private final PartGrid _partGrid;
    private int _xposition;
    private int _yposition;
    private final int _radius;
    private final List<Wire> _attachments = new LinkedList<>();

    public ProgramakerCustomBlockInputConnector(Part part, PartGrid partGrid,
                                                int inputConnectorCenterX, int inputConnectorCenterY,
                                                int inputConnectorRadius, ConnectorTypeInfo typeInfo) {
        _part = part;
        _partGrid = partGrid;
        _xposition = inputConnectorCenterX;
        _yposition = inputConnectorCenterY;
        _radius = inputConnectorRadius;
        _typeInfo = typeInfo;
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return ((Math.abs(x - _xposition) <= _radius)
                && (Math.abs(y - _yposition) <= _radius));
    }

    @Override
    public Moveable getMoveable() {
        return new Wire(this, _xposition, _yposition);
    }

    @Override
    public void unlink() {
        for (Wire wire : _attachments) {
            wire.unlink();
        }
    }

    @Override
    public void updatePosition(int x, int y) {
        _xposition = x;
        _yposition = y;

        for (Wire wire : _attachments){
            wire.moveEnd(x, y);
        }
    }

    public Object query(Object lastValue) {
        for (Wire linked : _attachments) {
            Object returned = linked.query(lastValue);
            if (returned != null) {
                return returned;
            }
        }

        return null;
    }

    @Override
    public int getX() {
        return _xposition;
    }

    @Override
    public int getY() {
        return _yposition;
    }

    public float getRadius() {
        return _radius;
    }

    @Override
    public void send(AnySignal signal) {
        _part.send(this, signal);
    }

    @Override
    public void addAttachment(Wire wire) {
        _attachments.add(wire);
    }

    @Override
    public Part getPart() {
        return _part;
    }

    @Override
    public String getId() {
        return _part.getConnectorId(this);
    }

    @Override
    public void drop(Wire wire) {
        Log.d("InputConnector", "Dropped wire " + wire);
    }

    @Override
    public void wireUnlinked(Wire wire) {
        _attachments.remove(wire);
    }

    public void touched() {
        // If it doesn't have wires connected, spawn a value block
        if (this._attachments.size() > 0) {
            return;
        }

        final PartGrid grid = _partGrid;

        if (grid instanceof CanvasView) {
            StaticValuePart value = new StaticValuePart(_partGrid,
                    this._xposition, this._yposition - STATIC_VALUE_SEPARATION,
                    this._typeInfo);
            AnyWire wire = value.getOutputWire();
            value.getValueOutput().drop(wire, this);
            ((CanvasView) grid).addPart(value);
        }
    }
}
