package com.codigoparallevar.minicards.parts.values;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.AsyncTask;
import android.text.InputType;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ProgressBar;

import com.codigoparallevar.minicards.R;
import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.parts.connectors.AnyRoundOutputConnector;
import com.codigoparallevar.minicards.parts.connectors.ConnectorTypeInfo;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartConnection;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.connectors.Wiring.AnyWire;
import com.codigoparallevar.minicards.types.connectors.input.InputConnector;
import com.codigoparallevar.minicards.types.connectors.output.OutputConnector;
import com.codigoparallevar.minicards.types.functional.Consumer;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.codigoparallevar.minicards.types.functional.Tuple3;
import com.codigoparallevar.minicards.types.wireData.WireDataType;
import com.codigoparallevar.minicards.ui_helpers.GetAsync;
import com.codigoparallevar.minicards.utils.Serializations;
import com.programaker.api.ProgramakerApi;
import com.programaker.api.data.ProgramakerCustomBlockArgument;
import com.programaker.api.data.ProgramakerCustomBlockArgumentValue;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

public class StaticValuePart implements Part {
    private static final int WIDTH_PADDING = 25;
    private static final int HEIGHT_PADDING = 25;
    private static final int MIN_HEIGHT = 100;
    private static final int MIN_WIDTH = 100;
    private static final String LogTag = "PM StaticValue";
    private static final int IO_RADIUS = 50;
    private static final int IO_PADDING = 20;

    private final String _id;
    private final AnyRoundOutputConnector _outputConnector;
    private final ConnectorTypeInfo _typeInfo;
    private final PartGrid _grid;
    private int _width;
    private int _height;
    private int _left;
    private int _top;
    private String _value = null;
    private String _valueId = null;

    public StaticValuePart(String id, PartGrid grid, int centerx, int centery, ConnectorTypeInfo typeInfo, String value, String valueId) {
        this._id = id;
        this._typeInfo = typeInfo;
        this._grid = grid;
        this._value = value;
        this._valueId = valueId;

        this.updateWidthHeight();

        this._left = centerx - this._width / 2;
        this._top = centery - this._height / 2;
        this._outputConnector = new AnyRoundOutputConnector(this, grid,
                this._left + _width / 2,
                this._top + _height, IO_RADIUS);
    }
    public StaticValuePart(String id, PartGrid grid, int centerx, int centery, ConnectorTypeInfo typeInfo, String value) {
        this(id, grid, centerx, centery, typeInfo, value, null);
    }

    public StaticValuePart(String id, PartGrid grid, int centerx, int centery, ConnectorTypeInfo typeInfo) {
        this(id, grid, centerx, centery, typeInfo, null);
    }

    public StaticValuePart(PartGrid grid, int centerx, int centery, ConnectorTypeInfo typeInfo) {
        this(UUID.randomUUID().toString(), grid, centerx, centery, typeInfo);
    }

    @Override
    public JSONObject serialize() throws JSONException {
        JSONObject serialized = new JSONObject();

        serialized.put("id", _id);
        serialized.put("center_x", _left + _width / 2);
        serialized.put("center_y", _top + _height / 2);
        serialized.put("value", _value == null ? JSONObject.NULL : _value);
        serialized.put("value_id", _valueId == null ? JSONObject.NULL : _valueId);

        JSONObject jsonTypeInfo = _typeInfo.serialize();
        serialized.put("type_info", jsonTypeInfo);

        serialized.put("on_string_output_connector", serializeConnectionEndpoints());

        return serialized;
    }

    private JSONArray serializeConnectionEndpoints() {
        JSONArray serializedData = new JSONArray();

        for (OutputConnector output : getOutputConnectors()) {
            JSONArray elements = new JSONArray();

            for (Tuple2<String, String> endpoint : (List<Tuple2<String, String>>) output.getConnectionEndpoints()) {
                elements.put(PartConnection.serializeToJson(endpoint.item1, endpoint.item2));
            }

            serializedData.put(elements);
        }
        return serializedData;
    }

    public static Tuple2<Part, List<PartConnection>> deserialize(PartGrid grid, JSONObject data) throws JSONException {
        String id = data.getString("id");
        int centerx = data.getInt("center_x");
        int centery = data.getInt("center_y");
        String value = Serializations.getString(data, "value");
        String valueId = Serializations.getString(data, "value_id");

        JSONObject jsonTypeInfo = data.getJSONObject("type_info");
        ConnectorTypeInfo typeInfo = ConnectorTypeInfo.deserialize(jsonTypeInfo);
        StaticValuePart part = new StaticValuePart(id, grid, centerx, centery, typeInfo, value, valueId);

        List<PartConnection> connections = new LinkedList<>();

        JSONArray allConnectorOuts = data.optJSONArray("on_string_output_connector");
        if (allConnectorOuts == null) {
            allConnectorOuts = new JSONArray();
        }

        for (int i = 0; i < allConnectorOuts.length(); i++) {
            JSONArray connectorOuts = allConnectorOuts.getJSONArray(i);
            OutputConnector connector = part.getOutputConnectors().get(i);

            for (int j = 0; j < connectorOuts.length(); j++) {
                connections.add(PartConnection.deserialize(
                        connector, connectorOuts.getJSONObject(j)));
            }
        }

        return new Tuple2<>(part, connections);
    }

    public AnyWire getOutputWire() {
        return this._outputConnector.getMoveable();
    }

    @Override
    public int get_left() {
        return _left;
    }

    @Override
    public int get_right() {
        return _left + _width;
    }

    @Override
    public int get_top() {
        return _top;
    }

    @Override
    public int get_bottom() {
        return _top + _height;
    }

    private void updateWidthHeight() {
        Paint p = getTextPaint();
        String message = getMessage();
        Rect bounds = new Rect();
        p.getTextBounds(message, 0, message.length(), bounds);

        this._height = Math.max(MIN_HEIGHT, bounds.height() + HEIGHT_PADDING * 2);
        int newWidth = Math.max(MIN_WIDTH, bounds.width() + WIDTH_PADDING * 2);

        if (this._width > 0) { // Re-center block
            this._left -= (newWidth - _width) / 2;
        }
        this._width = newWidth;
    }

    private String getMessage() {
        if (_value == null) {
            return "-";
        } else {
            return _value;
        }
    }

    private String getMessageForEdit() {
        ConnectorTypeInfo.Type type = _typeInfo.get_type();
        if (type == ConnectorTypeInfo.Type.INTEGER || type == ConnectorTypeInfo.Type.FLOAT) {
            if (_value == null) {
                return "0";
            }
            try {
                Integer.parseInt(_value);
                return _value;
            } catch(NumberFormatException ex) {
                Log.w(LogTag, "StaticPart (type=number), value found=" + _value, ex);
                return "0";
            }
        }
        else {
            if (_value == null) {
                return "";
            }
            return _value;
        }
    }

    private Paint getTextPaint() {
        Paint p = new Paint(Paint.ANTI_ALIAS_FLAG);
        p.setColor(Color.BLACK);
        p.setTextSize(50);
        return p;
    }

    @Override
    public void touched() {
        if (!(this._grid instanceof View)) {
            return; // Can't show dialog if can't retrieve context
        }

        final Context ctx = ((View) this._grid).getContext();
        ConnectorTypeInfo.Type type = this._typeInfo.get_type();
        if (type == ConnectorTypeInfo.Type.PULSE) {
            return; // Nothing to do here
        }

        switch (type) {
            case BOOLEAN:
                break;
            case INTEGER:
            case FLOAT: {
                final EditText input = new EditText(ctx);

                input.setText(getMessageForEdit());
                input.setInputType(InputType.TYPE_CLASS_NUMBER);

                dialogAskUserForValue(ctx, input,
                        () -> {
                            _value = input.getText().toString();
                        });
                break;
            }
            default:
            case ANY:
            case UNKNOWN:
            case STRING: {
                final EditText input = new EditText(ctx);

                input.setText(getMessageForEdit());
                input.setInputType(InputType.TYPE_CLASS_TEXT);
                dialogAskUserForValue(ctx, input,
                        () -> {
                            _value = input.getText().toString();
                        });
                break;
            }
            case ENUM:
                ProgramakerCustomBlockArgument blockArg = _typeInfo.getBlockArgument();
                String callback = null;
                if (blockArg != null) {
                    callback = blockArg.getCallback();
                    if (callback.equals("null")) { // TODO: Fix this on server
                        callback = null;
                    }
                }

                if (callback == null) {
                    new AlertDialog.Builder(ctx)
                            .setMessage(R.string.error_recovering_allowed_values)
                            .create()
                            .show();
                    break;
                }

                ProgramakerApi api = _grid.getApi();
                Dialog loadingDialog = createLoadingDialog(ctx);
                if (!api.hasCachedAllowedValues(blockArg)) {
                    loadingDialog.show();
                }

                new GetAsync<List<ProgramakerCustomBlockArgumentValue>>().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,
                        new Tuple3<>(
                                () -> api.fetchAllowedValues(blockArg),
                                results -> {
                                    Log.i(LogTag, "Result: " + results);
                                    loadingDialog.cancel();

                                    String[] names = new String[results.size()];
                                    int i = 0;
                                    for (ProgramakerCustomBlockArgumentValue value : results) {
                                        names[i] = value.getName();
                                        i++;
                                    }

                                    Dialog d = createItemDialog(ctx, names, idx -> {
                                        StaticValuePart.this._value = results.get(idx).getName();
                                        StaticValuePart.this._valueId = results.get(idx).getId();
                                        StaticValuePart.this._grid.update();
                                    });
                                    d.show();
                                },
                                ex -> {
                                    Log.e(LogTag, "Error retrieving values: " + ex, ex);
                                    new AlertDialog.Builder(ctx)
                                            .setMessage(R.string.error_recovering_allowed_values)
                                            .create()
                                            .show();
                                }
                        ));

                break;
        }
    }

    private Dialog createItemDialog(Context ctx, CharSequence[] itemNames, Consumer<Integer> onSelected) {
        final AlertDialog.Builder builder = new AlertDialog.Builder(ctx);

        builder.setTitle(R.string.set_value);

        final ProgressBar progressBar = new ProgressBar(ctx);
        progressBar.setIndeterminate(true);
        builder.setItems(itemNames, (DialogInterface.OnClickListener) (dialog, which) -> {
            try {
                onSelected.apply(which);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        AlertDialog dialog = builder.create();
        return dialog;
    }

    private Dialog createLoadingDialog(Context ctx) {
        final AlertDialog.Builder builder = new AlertDialog.Builder(ctx);

        builder.setTitle(R.string.loading_available_values);

        final ProgressBar progressBar = new ProgressBar(ctx);
        progressBar.setIndeterminate(true);
        builder.setView(progressBar);
        AlertDialog dialog = builder.create();
        return dialog;
    }

    private void dialogAskUserForValue(Context ctx, EditText input, Runnable onAccept) {
        final AlertDialog.Builder builder = new AlertDialog.Builder(ctx);

        builder.setTitle(R.string.set_value);
        builder
                .setPositiveButton(R.string.ok_accept_changes, (dialog, which) -> {
                    onAccept.run();
                    this.updateWidthHeight();
                    ((View) this._grid).invalidate();
                })
                .setNegativeButton(R.string.cancel_discard_changes, (dialog, which) -> {
                    // No change
                })
                .setView(input);

        AlertDialog dialog = builder.create();
        dialog.show();

    }

    @Override
    public List<InputConnector> getInputConnectors() {
        // There is not going to be one
        return Collections.emptyList();
    }

    @Override
    public List<OutputConnector> getOutputConnectors() {
        return new LinkedList<OutputConnector>() {{
            add(_outputConnector);
        }};
    }

    public AnyRoundOutputConnector getValueOutput() {
        return _outputConnector;
    }

    @Override
    public void send(InputConnector inputConnector, WireDataType signal) {

    }

    @Override
    public String get_id() {
        return _id;
    }

    @Override
    public InputConnector getConnectorWithId(String inputConnectorId) {
        return null; // No inputs
    }

    @Override
    public String getConnectorId(InputConnector inputConnector) {
        return null; // No inputs
    }

    @Override
    public void resume() {
        // Nothing to do
    }

    @Override
    public void pause() {
        // Nothing to do
    }

    @Override
    public Object query(Object lastValue) {
        if (this._valueId != null) {
            return this._valueId;
        }

        return this._value;
    }

    @Override
    public void draw(ScrolledCanvas canvas, boolean devMode) {
        this.updateWidthHeight(); // TODO: Remove after calculations stabilize

        if (!devMode) {
            return; // Logic block, don't show on user-mode
        }

        drawConnectors(canvas);
        drawWires(canvas, devMode);

        Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        paint.setColor(Color.parseColor("#E3FCFC"));
        canvas.drawRect(
                new Rect(get_left(), get_top(),
                        get_right(), get_bottom()),
                paint);

        Paint textPaint = getTextPaint();
        canvas.drawText(getMessage(),
                get_left() + WIDTH_PADDING,
                get_bottom() - HEIGHT_PADDING,
                textPaint);
    }

    private void drawConnectors(ScrolledCanvas canvas) {
        Paint outerOutputConnectorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        outerOutputConnectorPaint.setColor(_typeInfo.getOuterColor());

        canvas.drawCircle(
                _outputConnector.getX(),
                _outputConnector.getY(),
                _outputConnector.getRadius(),
                outerOutputConnectorPaint);

        Paint innerOutputConnectorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        innerOutputConnectorPaint.setColor(_typeInfo.getInnerColor());

        canvas.drawCircle(
                _outputConnector.getX(),
                _outputConnector.getY(),
                _outputConnector.getRadius() / 2,
                innerOutputConnectorPaint);
    }

    private void drawWires(ScrolledCanvas canvas, boolean devMode) {
        _outputConnector.drawWires(canvas, devMode);
    }

    @Override
    public void moveEnd(int x, int y) {
        _left = x - _width / 2;
        _top = y - _height / 2;
        this._outputConnector.updatePosition(
                this._left + _width / 2,
                this._top + _height);
    }

    @Override
    public void drop(int x, int y) {
        moveEnd(x, y);
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return ((x >= this.get_left()) && (x <= this.get_right())
                && (y >= this.get_top()) && (y <= this.get_bottom()));
    }

    @Override
    public Moveable getMoveable() {
        return this;
    }

    @Override
    public void unlink() {
        // Nothing to do
        _outputConnector.unlink();
    }
}
