package com.codigoparallevar.minicards.toolbox;

import android.content.Context;
import android.graphics.Canvas;
import android.view.View;

import com.codigoparallevar.minicards.PartInstantiator;
import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.SignalListenerManager;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.Selectable;
import com.codigoparallevar.minicards.types.connectors.input.AnyInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.BooleanInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.ImageInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.SignalInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.StringInputConnector;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.programaker.api.ProgramakerApi;

class PartShowcaseView extends View implements PartGrid {
    private final Context ctx;
    private final PartInstantiator instantiator;
    private final Part part;
    private final int partWidth;
    private Tuple2<Integer, Integer> center = new Tuple2<>(0, 0);

    public PartShowcaseView(Context ctx, PartInstantiator instantiator) {
        super(ctx);

        this.ctx = ctx;
        this.instantiator = instantiator;
        this.part = instantiator.build(this);
        this.part.pause();
        int height = this.part.get_bottom() - this.part.get_top();
        partWidth = this.part.get_right() - this.part.get_left();

        this.setMinimumHeight(height * 2);
        this.setMinimumWidth(partWidth);
    }

    @Override
    public void onDraw(Canvas canvas) {
        int centerx = - Math.max(canvas.getWidth(), partWidth) / 2;
        int centery = - canvas.getHeight() / 2;
        this.center = new Tuple2<>(centerx, centery);
        ScrolledCanvas scrolledCanvas = new ScrolledCanvas(canvas, getCenteredOn());
        this.part.draw(scrolledCanvas, true);
    }

    // Most interactive functionalities are not implemented, as it's not needed for the showcase
    @Override
    public Selectable getPartOn(int x, int y) {
        return null;
    }

    @Override
    public ProgramakerApi getApi() {
        return null;
    }

    @Override
    public SignalListenerManager getListenerManager() {
        return null;
    }

    @Override
    public SignalInputConnector getSignalInputConnectorOn(int x, int y) {
        return null;
    }

    @Override
    public BooleanInputConnector getBooleanInputConnectorOn(int x, int y) {
        return null;
    }

    @Override
    public AnyInputConnector getAnyInputConnectorOn(int x, int y) {
        return null;
    }

    @Override
    public StringInputConnector getStringInputConnectorOn(int xEnd, int yEnd) {
        return null;
    }

    @Override
    public ImageInputConnector getImageInputConnectorOn(int xEnd, int yEnd) {
        return null;
    }

    @Override
    public Tuple2<Integer, Integer> getCenteredOn() {
        return center;
    }
    @Override

    public void update() {

    }
}
