package com.codigoparallevar.minicards.toolbox;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Build;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;

import com.codigoparallevar.minicards.CanvasView;
import com.codigoparallevar.minicards.PartInstantiator;
import com.codigoparallevar.minicards.parts.android.CameraStreamer;
import com.codigoparallevar.minicards.parts.buttons.RoundButton;
import com.codigoparallevar.minicards.parts.logic.Ticker;
import com.codigoparallevar.minicards.parts.logic.Toggle;
import com.codigoparallevar.minicards.parts.samples.ColorBox;
import com.codigoparallevar.minicards.parts.strings.ConvertToString;
import com.codigoparallevar.minicards.parts.viewers.ImageFrame;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.functional.Consumer;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.codigoparallevar.minicards.utils.Box;
import com.programaker.api.data.ProgramakerBridgeInfo;
import com.programaker.api.data.ProgramakerCustomBlock;
import com.programaker.api.data.api_results.ProgramakerBridgeCustomBlockResult;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class PartsHolder {
    private final Context context;

    private final static List<Tuple2<String, PartInstantiator>> GuiParts =
            new Vector<Tuple2<String, PartInstantiator>>(){{
                add(new Tuple2<>("Round button", RoundButton.getInstantiator()));
                add(new Tuple2<>("Red/Green box", ColorBox.getInstantiator()));
                add(new Tuple2<>("Image frame", ImageFrame.getInstantiator()));
            }};

    private final static List<Tuple2<String, PartInstantiator>> AndroidParts =
            new Vector<Tuple2<String, PartInstantiator>>(){{
                add(new Tuple2<>("Camera", CameraStreamer.getInstantiator()));
            }};

    private final static List<Tuple2<String, PartInstantiator>> MiscParts =
            new Vector<Tuple2<String, PartInstantiator>>(){{
                add(new Tuple2<>("Ticker", Ticker.getInstantiator()));
                add(new Tuple2<>("Toggle", Toggle.getInstantiator()));
            }};

    private final static List<Tuple2<String, PartInstantiator>> DebuggingParts =
            new Vector<Tuple2<String, PartInstantiator>>(){{
                add(new Tuple2<>("ToString", ConvertToString.getInstantiator()));
            }};


    private final List<PartCategory> Categories = new Vector<PartCategory>() {{
        add(new PartCategory("GUI elements", GuiParts));
        add(new PartCategory("Android", AndroidParts));
        add(new PartCategory("Misc", MiscParts));
        add(new PartCategory("Debugging", DebuggingParts));
    }};

    private Map<String, ProgramakerBridgeInfo> bridgeInfoMap;
    private final static String LogTag = "PartsHolder";

    public PartsHolder(Context context) {
        this.context = context;
    }

    public void openAddPartModal(final CanvasView canvasView) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);

        // HACK : Nasty trick to make the accordion be able to close a dialog that still doesn't exist
        Box<Dialog> openedDialog = new Box<>();
        Consumer<Void> onComplete = _void -> {
            if (openedDialog.get() == null) {
                Log.e(LogTag, "Expected dialog to be opened already");
            }
            else {
                openedDialog.get().cancel();
            }
        };

        View categoryAccordion = generateCategoryAccordion(canvasView, onComplete);

        builder.setTitle("Choose part")
                .setView(categoryAccordion);

        Dialog dialog = builder.create();
        openedDialog.set(dialog);
        dialog.show();
    }

    private View generateCategoryAccordion(final CanvasView canvasView, Consumer<Void> signalComplete) {
        ScrollView view = new ScrollView(context);
        LinearLayout layout = new LinearLayout(context);
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.setDividerPadding(10);
        layout.setShowDividers(LinearLayout.SHOW_DIVIDER_BEGINNING | LinearLayout.SHOW_DIVIDER_MIDDLE);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            layout.setDividerDrawable(context.getDrawable(android.R.drawable.divider_horizontal_bright));
        }

        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT
        );

        for (int i = 0; i < Categories.size(); i++) {
            PartCategory category = Categories.get(i);

            LinearLayout categoryRow = new LinearLayout(context);
            LinearLayout showcases = new LinearLayout(context);

            TextView categoryName = new TextView(context); //Creating Button

            // Add category name
            categoryName.setId(i); //Setting Id for using in future
            categoryName.setText(category.getName()); //Setting text
            categoryName.setTextSize(15); //Text Size
            categoryName.setPadding(50, 50, 50, 50); //padding
            categoryName.setLayoutParams(params); //Setting Layout Params
            categoryName.setTextColor(Color.parseColor("#000000")); //Text Color
            categoryName.setBackgroundColor(Color.parseColor("#ffffff"));
            categoryName.setGravity(Gravity.CENTER);
            categoryName.setClickable(true);
            categoryName.setOnClickListener(v -> {
                if (showcases.getVisibility() == View.GONE) {
                    showcases.setVisibility(View.VISIBLE);
                }
                else {
                    showcases.setVisibility(View.GONE);
                }
            });

            showcases.setVisibility(View.GONE);

            showcases.setBackgroundColor(Color.parseColor("#BBBBBB"));
            showcases.setOrientation(LinearLayout.VERTICAL);
            showcases.setDividerPadding(10);
            showcases.setShowDividers(LinearLayout.SHOW_DIVIDER_BEGINNING | LinearLayout.SHOW_DIVIDER_MIDDLE);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                showcases.setDividerDrawable(context.getDrawable(android.R.drawable.divider_horizontal_bright));
            }

            // Add showcases
            int partHeights = 0;
            for (Tuple2<String, PartInstantiator> entry : category.getParts()) {
                PartShowcaseView showcase = new PartShowcaseView(context, entry.item2);

                showcase.setClickable(true);
                showcase.setOnClickListener(v -> {
                    Part part = entry.item2.build(canvasView);
                    canvasView.addPart(part);
                    try {
                        signalComplete.apply(null);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
                partHeights += showcase.getHeight();

                showcases.addView(showcase);
            }
            showcases.setMinimumHeight(partHeights);

            // Tie everything
            categoryRow.setOrientation(LinearLayout.VERTICAL);
            categoryRow.addView(categoryName);
            showcases.setOrientation(LinearLayout.VERTICAL);
            categoryRow.addView(showcases);

            layout.addView(categoryRow);
        }

        view.addView(layout);
        return view;
    }

    public void addCustomBlocks(List<ProgramakerBridgeInfo> bridgeInfo, List<ProgramakerBridgeCustomBlockResult> customBlocks) {
        Map<String, ProgramakerBridgeInfo> bridgeInfoMap = new LinkedHashMap<>();
        for (ProgramakerBridgeInfo info : bridgeInfo) {
            bridgeInfoMap.put(info.getId(), info);
        }

        this.bridgeInfoMap = bridgeInfoMap;

        for (ProgramakerBridgeCustomBlockResult res : customBlocks) {

            List<Tuple2<String, PartInstantiator>> parts = new LinkedList<>();
            for (ProgramakerCustomBlock block : res.getBlocks()) {
                parts.add(new Tuple2<>(block.getMessage(), new ProgramakerCustomBlockInstantiator(block)));
            }

            if (bridgeInfoMap.containsKey(res.getBridge_id())) {
                Categories.add(new PartCategory(bridgeInfoMap.get(res.getBridge_id()).getName(), parts));
            }
            else {
                Categories.add(new PartCategory("Bridge ID="+res.getBridge_id(), parts));
            }
        }
    }
}
