package com.codigoparallevar.minicards.types.connectors.input;

import androidx.annotation.NonNull;

import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.connectors.Wiring.Wire;
import com.codigoparallevar.minicards.types.connectors.output.BooleanOutputConnector;
import com.codigoparallevar.minicards.types.connectors.output.OutputConnector;
import com.codigoparallevar.minicards.types.connectors.output.SignalOutputConnector;
import com.codigoparallevar.minicards.types.connectors.output.StringOutputConnector;
import com.codigoparallevar.minicards.types.wireData.AnySignal;
import com.codigoparallevar.minicards.types.wireData.BooleanSignal;
import com.codigoparallevar.minicards.types.wireData.Signal;
import com.codigoparallevar.minicards.types.wireData.StringSignal;


public abstract class AnyInputConnector implements InputConnector<AnySignal, AnyInputConnector> {

    public SignalInputConnector ToSignalInputConnector() {
        return new WrapAsSignal(this);
    }

    public BooleanInputConnector ToBooleanInputConnector(){
        return new WrapAsBoolean(this);
    }

    public StringInputConnector ToStringInputConnector(){
        return new WrapAsString(this);
    }

    @NonNull
    public InputConnector matchType(@NonNull OutputConnector outputConnector) {
        if (outputConnector instanceof SignalOutputConnector) {
            return this.ToSignalInputConnector();
        }
        else if (outputConnector instanceof BooleanOutputConnector){
            return this.ToBooleanInputConnector();
        }
        else if (outputConnector instanceof StringOutputConnector){
            return this.ToStringInputConnector();
        }

        return this;
    }

    private class WrapAsSignal extends WrapAs implements SignalInputConnector {
        WrapAsSignal(AnyInputConnector baseInputConnector) {
            super(baseInputConnector);
        }

        @Override
        public void send(Signal data) {
            baseInputConnector.send(new AnySignal(data.get()));
        }
    }

    private class WrapAsBoolean extends WrapAs implements BooleanInputConnector {
        WrapAsBoolean(AnyInputConnector baseInputConnector) {
            super(baseInputConnector);
        }

        @Override
        public void send(BooleanSignal data) {
            baseInputConnector.send(new AnySignal(data.get()));
        }
    }

    private class WrapAsString extends WrapAs implements StringInputConnector {
        WrapAsString(AnyInputConnector baseInputConnector) {
            super(baseInputConnector);
        }

        @Override
        public void send(StringSignal data) {
            baseInputConnector.send(new AnySignal(data.get()));
        }
    }

    private abstract class WrapAs {
        final AnyInputConnector baseInputConnector;

        WrapAs(AnyInputConnector baseInputConnector) {
            this.baseInputConnector = baseInputConnector;
        }

        public boolean containsPoint(int x, int y) {
            return baseInputConnector.containsPoint(x, y);
        }

        public Moveable getMoveable() {
            return baseInputConnector.getMoveable();
        }

        public void unlink() {
            baseInputConnector.unlink();
        }

        public void drop(Wire wire) {
            baseInputConnector.drop(wire);
        }

        public void wireUnlinked(Wire wire) {
            baseInputConnector.wireUnlinked(wire);
        }

        public void updatePosition(int x, int y) {
            baseInputConnector.updatePosition(x, y);
        }

        public int getX() {
            return baseInputConnector.getX();
        }

        public int getY() {
            return baseInputConnector.getY();
        }

        public Part getPart() {
            return baseInputConnector.getPart();
        }

        public String getId() {
            return baseInputConnector.getId();
        }

        public void addAttachment(Wire wire) {
            baseInputConnector.addAttachment(wire);
        }
    }
}
