package com.codigoparallevar.minicards.utils;

import android.util.Log;

import org.jetbrains.annotations.NotNull;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;
import java.util.Map;

public class Serializations {

    private static final String LogTag = "Serialization";

    public static JSONArray serialize(List<Map<String,String>> data) {
        JSONArray array = new JSONArray();
        for (Map<String, String> dict : data) {
            array.put(new JSONObject(dict));
        }

        return array;
    }

    @NotNull
    public static Object blindSerialize(Object obj) {
        if (obj == null) {
            return JSONObject.NULL;
        }
        // No serialization needed
        else if (obj.getClass().isPrimitive() ||
                obj instanceof String ||
                obj instanceof JSONObject ||
                obj instanceof JSONArray) {

            return obj;
        }

        try {
            return _blindSerialize(obj);
        }
        catch (Exception ex) {
            Log.w(LogTag, "Error serialization: " + obj, ex);
            return obj;
        }
    }

    private static Object _blindSerialize(Object obj) throws JSONException {
        if (obj instanceof Map) {
            Map<?,?> objMap = (Map<?,?>) obj;

            JSONObject map = new JSONObject();

            for (Map.Entry<?,?> entry : objMap.entrySet()) {
                map.put(entry.getKey().toString(), blindSerialize(entry.getValue()));
            }

            return map;
        }
        else if (obj instanceof Iterable) {
            Iterable objIt = (Iterable) obj;

            JSONArray arr = new JSONArray();
            for (Object val : objIt) {
                arr.put(blindSerialize(val));
            }
            return arr;
        }
        else {
            // Unknown case
            return obj;
        }
    }

    public static String getString(JSONObject map, String key) {
        if (!map.has(key)) {
            return null;
        }
        if (map.opt(key) == JSONObject.NULL) {
            return null;
        }
        try {
            return map.getString(key);
        } catch (JSONException e) {
            return null;
        }
    }
}
