package com.programaker.api

import android.util.Log
import com.google.gson.Gson
import okhttp3.*
import okio.ByteString
import org.json.JSONException
import java.nio.charset.Charset
import java.util.concurrent.TimeUnit

class ProgramakerListeningChannel(
        private val bridgeId: String,
        private val key: String,
        private val token: String,
        private val url: String,
        private val listener: ProgramakerSignalListener,
        private var onDisconnect: Runnable
): WebSocketListener() {
    private var webSocket: WebSocket? = null
    private val utf8: Charset = Charset.forName("UTF-8")
    private val gson = Gson()

    private val LogTag: String = "PM-ListeningChannel"
    private val PING_PERIOD_MILLIS: Long = 15000 // 15seconds

    fun start() {
        val client: OkHttpClient = OkHttpClient.Builder()
                .pingInterval(PING_PERIOD_MILLIS, TimeUnit.MILLISECONDS)
                .readTimeout(0, TimeUnit.MILLISECONDS)
                .build()

        val request: Request = Request.Builder()
                .url(url)
                .addHeader("Authorization", token)
                .build()
        client.newWebSocket(request, this)

        // Trigger shutdown of the dispatcher's executor so this process can exit cleanly.
        client.dispatcher.executorService.shutdown()
    }

    // Websocket management
    override fun onOpen(webSocket: WebSocket, response: Response) {
        this.webSocket = webSocket
    }

    override fun onMessage(webSocket: WebSocket, bytes: ByteString) {
        onMessage(webSocket, bytes.string(utf8))
    }

    override fun onMessage(webSocket: WebSocket, text: String) {
        Log.d(LogTag, "Message: $text")
        val json = gson.fromJson(text, HashMap::class.java)
        if (json == null) {
            this.onFailure(webSocket, JSONException("Error decoding: $text"), null)
        }
        else {
            this.listener.onNewSignal(bridgeId, key, json)
        }
    }

    override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
        webSocket.close(1000, null)
        Log.i(LogTag, "Closing bridge socket {code=$code, reason=$reason}")
        onDisconnect.run()
    }

    override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
        Log.e(LogTag, "Error: $t", t)
        onDisconnect.run()
    }

    fun stop() {
        onDisconnect = Runnable {  } // Skip disconnection procedure
        webSocket?.close(1000, null)
        webSocket = null
    }
}
